package com.nartosoft.picuploader;
/*
 * Doesn't contain implementation for watermark, I'm too lazy and people should use FileConfiguration anyway... (Michael)
 */
import java.awt.Color;
import java.awt.Font;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author eb
 * 
 * Provides easy access to the commandline parameters of PicUplader.
 */
public class ArgumentConfiguration implements Configuration {
	public static final String PROXY_COMMAND = "-p";

	public static final String LOGFILE_COMMAND = "-f";

	public static final String LOGFILEFORMAT_COMMAND = "-ff";

	public static final String TEMPLATE_COMMAND = "-t";

	public static final String VERBOSE_SWITCH = "-v";

	public static final String HTMLFILE_COMMAND = "-h";

	public static final String SUMMARY_COMMAND = "-summary";

	public static final String SOUND_COMMAND = "-sound";

	public static final String AUTOSCALE_COMMAND = "-autoscale";
 
	public static final String TRAYICON_COMMAND = "-trayicon";
	
	public static final String SCREENSHOT_COMMAND = "-screenshot";
	
	public static final String SHOWSCREENSHOTDIALOG_SWITCH = "-showgui";
	
	private static final List COMMANDS = Arrays.asList( new String[] { 
			PROXY_COMMAND, 
			LOGFILE_COMMAND, 
			LOGFILEFORMAT_COMMAND, 
			TEMPLATE_COMMAND, 
			HTMLFILE_COMMAND, 
			SUMMARY_COMMAND, 
			SOUND_COMMAND,
			AUTOSCALE_COMMAND,
			TRAYICON_COMMAND,
			SCREENSHOT_COMMAND			
	});
	private static final List SWITCHES = Arrays.asList( new String[] { VERBOSE_SWITCH,SHOWSCREENSHOTDIALOG_SWITCH } );
	
	private final String[] args;

	private final Map commandLookup;

	private final List switchLookup;

	public ArgumentConfiguration(String[] args) {
		this.args = (String[]) args.clone(); // keep a copy
		commandLookup = createCommandLookup();
		switchLookup = createSwitchLookup();
	}

	/**
	 * @return A list containing all switch instructions (parameterless options).
	 */
	private List createSwitchLookup() {
		List lookup = new ArrayList();
		for (int i = 0; i < args.length; i++) {
			if (isSwitch(args[i])) {
				lookup.add(args[i].toLowerCase());
			}
		}
		return lookup;
	}

	/**
	 * @return A map containing all commands (parameterized options) mapped to
	 *         their parameters.
	 */
	private Map createCommandLookup() {
		Map lookup = new HashMap();
		for (int i = 0; i < args.length; i++) {
			if (isCommand(args[i]) && i != args.length - 1) {
				lookup.put(args[i].toLowerCase(), args[i + 1]);
			}
		}
		return lookup;
	}

	/**
	 * @return A list containing all the file names in the arguments.
	 */
	public List getFileNames() {
		List list = new ArrayList();
		for (int i = 0; i < args.length; i++) {
			if (isSwitch(args[i])) {
				// ignore
			}
			else if (isCommand(args[i])) {
				i++; // skip parameter
			}
			else {
				list.add(args[i]);
			}
		}
		return list;
	}

	public boolean isProxySet() {
		return commandLookup.containsKey(PROXY_COMMAND);
	}

	public boolean isLogFileSet() {
		return commandLookup.containsKey(LOGFILE_COMMAND);
	}

	public boolean isLogFileFormatSet() {
		return commandLookup.containsKey(LOGFILEFORMAT_COMMAND);
	}

	public boolean isHtmlFileSet() {
		return commandLookup.containsKey(HTMLFILE_COMMAND);
	}

	public boolean isTemplateSet() {
		return commandLookup.containsKey(TEMPLATE_COMMAND);
	}

	public boolean isSummarySet() {
		return commandLookup.containsKey(SUMMARY_COMMAND);
	}

	public boolean isSoundSet() {
		return commandLookup.containsKey(SOUND_COMMAND);
	}

	public boolean isAutoscaleSet() {
		return commandLookup.containsKey(AUTOSCALE_COMMAND);
	}

	public boolean isTrayiconSet() {
		return commandLookup.containsKey(TRAYICON_COMMAND);
	}
	
	public boolean isScreenshotSet() {
		return commandLookup.containsKey(SCREENSHOT_COMMAND);
	}
	
	public boolean isShowGUISet() {
		return switchLookup.contains(SHOWSCREENSHOTDIALOG_SWITCH);
	}
	
	public int getMaxFileSize() {
		if (isAutoscaleSet()) {
			return Integer.parseInt((String) commandLookup.get(AUTOSCALE_COMMAND));
		}
		else {
			return -1;
		}
	}

	public String getSoundFile() {
		return (String) commandLookup.get(SOUND_COMMAND);
	}

	public boolean isVerboseSet() {
		return switchLookup.contains(VERBOSE_SWITCH);
	}

	public String getProxy() {
		return (String) commandLookup.get(PROXY_COMMAND);
	}

	public String getLogFileName() {
		return (String) commandLookup.get(LOGFILE_COMMAND);
	}

	public String getSummaryTime() {
		return (String) commandLookup.get(SUMMARY_COMMAND);
	}

	public String getTrayiconFile() {
		return (String)commandLookup.get(TRAYICON_COMMAND);
	}
	
	public int getScreenshotDelay() {
		String s = (String)commandLookup.get(SCREENSHOT_COMMAND);
		if(s!=null) {
			return Integer.parseInt(s);
		}
		else {
			return 0;
		}
	}
	
	private String getLogFileFormat() {
		if (isLogFileFormatSet()) {
			return ((String) commandLookup.get(LOGFILEFORMAT_COMMAND)).toLowerCase();
		}
		else {
			return "";
		}
	}

	public boolean isImageFileLoggingSet() {
		return getLogFileFormat().indexOf('i') >= 0;
	}

	public boolean isUrlLoggingSet() {
		return getLogFileFormat().indexOf('u') >= 0;
	}

	public boolean isErrorLoggingSet() {
		return getLogFileFormat().indexOf('e') >= 0;
	}

	public String getTemplateFileName() {
		return (String) commandLookup.get(TEMPLATE_COMMAND);
	}

	public String getHtmlFileName() {
		return (String) commandLookup.get(HTMLFILE_COMMAND);
	}

	/**
	 * Returns the template string read from the template file, or null if either
	 * no template file was set or an error occured while reading it.
	 */
	public String getTemplate() {
		if (!isTemplateSet()) {
			return null;
		}
		return Utils.readFile(getTemplateFileName());
	}

	private boolean isCommand(String commandName) {
		return COMMANDS.contains(commandName.toLowerCase());
	}

	private boolean isSwitch(String switchString) {
		return SWITCHES.contains(switchString.toLowerCase());
	}

	public Color getWatermarkColor() {
		// TODO Auto-generated method stub
		return null;
	}

	public Font getWatermarkFont() {
		// TODO Auto-generated method stub
		return null;
	}

	public String getWatermarkText() {
		// TODO Auto-generated method stub
		return null;
	}

	public int getWatermarkXor() {
		// TODO Auto-generated method stub
		return 0;
	}

	public int getWatermarkYor() {
		// TODO Auto-generated method stub
		return 0;
	}

	public boolean isWatermarkSet() {
		// TODO Auto-generated method stub
		return false;
	}
}
