package com.nartosoft.picuploader;

import java.awt.Color;
import java.awt.Font;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

public class PicUploader {
	private static final int MAX_FILESIZE = 681984;

	private static final List VALID_EXTENSIONS = Arrays.asList(new String[] {
			"jpg", "jpeg", "png", "bmp" });

	private final EventDispatcher events;

	public PicUploader(EventDispatcher events) {
		this.events = events;
	}

	/**
	 * Uploads all files specified in fileNames.
	 * 
	 * @throws IOException
	 */
	public void uploadFiles(List fileNames, boolean autoScale, int maxFileSize, Configuration options)
			throws IOException {
		for (Iterator it = fileNames.iterator(); it.hasNext();) {
			uploadFile((String) it.next(), autoScale, maxFileSize, options);
		}
		events.notifyAllDone();
	}

	/**
	 * Uploads the specified file to 666kb.com and writes the resulting url (or
	 * error message) to the EventWriter.
	 */
	private void uploadFile(String fileName, boolean autoScale, int maxFileSize, Configuration options)
			throws IOException {
		String tmpFileName = fileName;
		File file = new File(tmpFileName);
		String fileToUpload = tmpFileName;
		

		// do some checks if file is valid
		if (file.isDirectory() || !file.exists()) {
			events.notifyError("invalid file: " + tmpFileName);
			return;
		}

		final String fileExtension = getFileExtension(tmpFileName);
		if (VALID_EXTENSIONS.contains(fileExtension)) {
			events.notifyError("invalid image type: " + tmpFileName);
			return;
		}

		if (fileToUpload != null&&options.isWatermarkSet()) {
			try {
				fileToUpload = Watermark.addWatermark(new File(fileToUpload), options.getWatermarkText(), 
						                              options.getWatermarkColor(), options.getWatermarkXor(), 
						                              options.getWatermarkYor(),options.getWatermarkFont());
				file = new File(fileToUpload);
				}catch(Exception e) {
					events.notifyError("error adding watermark: " + e.getMessage());
				}
		}

		// what if the maximum file size of 666kb.com is changed to something bigger
		// than 666kb?
		if (file.length() > MAX_FILESIZE && autoScale) {
			// file is too big, scale it down and save the scaled down version
			fileToUpload = PictureScaler.scaleImage(fileToUpload, maxFileSize);
		}
		else if (file.length() > MAX_FILESIZE && !autoScale) {
			return;
		}

		if(fileToUpload!=null) {
			final ClientHttpRequest request = createHttpRequest(fileToUpload);
			final InputStream is = request.post();
			final BufferedReader br = new BufferedReader(new InputStreamReader(is));
			String searchExtension = fileExtension;
			if(fileExtension.equalsIgnoreCase(".jpeg")) {
				searchExtension = ".jpg";
			}
			final String url = getUrlFromReply(searchExtension, br);
			if (url == null) {
				events.notifyError("url not found in HTTP reply");
			}
			else {
				events.notifyUpload(fileToUpload, url);
			}
		}
		else {
			events.notifyError("fileToUpload is null");
		}
	}

	/**
	 * Returns the lower-case file extension (all characters after the last '.'
	 * character in fileName) or an empty string if no extension is found.
	 */
	private String getFileExtension(String fileName) {
		int extensionStart = fileName.lastIndexOf('.');
		if (extensionStart == -1) {
			return "";
		}
		else {
			return fileName.substring(extensionStart).toLowerCase();
		}
	}

	/**
	 * Creates an html request for 666kb.com containing the image that is to be
	 * uploaded.
	 */
	private ClientHttpRequest createHttpRequest(String fileName)
			throws MalformedURLException, IOException {
		final URL url = new URL("http://www.666kb.com/u.php");
		final ClientHttpRequest request = new ClientHttpRequest(url);
		request.setParameter("f", new File(fileName));
		request.setParameter("MAX_FILE_SIZE", Integer.toString(MAX_FILESIZE));
		return request;
	}

	/**
	 * Tries to find the url to the uploaded file in the returned html page.
	 */
	private String getUrlFromReply(final String fileExtension,
			final BufferedReader reply) throws IOException {
		try {
			String s = reply.readLine();
			while (s != null) {
				int pos1 = s.lastIndexOf("http");
				if (pos1 != -1) {
					int pos2 = s.toLowerCase().lastIndexOf(fileExtension);
					if (pos2 != -1) {
						return s.substring(pos1, pos2 + fileExtension.length());
					}
				}
				s = reply.readLine();
			}
		}
		finally {
			reply.close();
		}
		return null;
	}
}