package com.nartosoft.picuploader;

import java.awt.Dimension;
import java.awt.GraphicsEnvironment;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Properties;

import javax.imageio.ImageIO;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JScrollPane;

/**
 * @author eb
 * 
 * Builds a PicUploader depending on the arguments passed on commandline and
 * uses it to upload the provided images.
 */
public class PicUploaderRunner {
	private static final String TEMPLATE_URL_PLACEHOLDER = "%s";

	private static final String DEFAULT_TEMPLATE = TEMPLATE_URL_PLACEHOLDER;

	private static EventDispatcher events;

	private static LogfileWriter createLogfileWriter(Configuration options)
			throws IOException {
		if (options.isLogFileFormatSet()) {
			return new LogfileWriter(options.getLogFileName(), options
					.isImageFileLoggingSet(), options.isUrlLoggingSet(), options
					.isErrorLoggingSet());
		}
		else {
			// use default logfile format: log all available data
			return new LogfileWriter(options.getLogFileName(), true, true, true);
		}
	}

	private static HtmlFileWriter createHtmlFileWriter(Configuration options)
			throws IOException {
		if (options.getHtmlFileName() == null) {
			return null;
		}
		return new HtmlFileWriter(options.getHtmlFileName());
	}

	private static String createClipboardTemplate(Configuration options) {
		if (!options.isTemplateSet()) {
			return DEFAULT_TEMPLATE;
		}
		final String template = options.getTemplate();
		if (template == null || template.indexOf(TEMPLATE_URL_PLACEHOLDER) == -1) {
			events.notifyError("invalid template, must contain url placeholder "
					+ TEMPLATE_URL_PLACEHOLDER);
			return DEFAULT_TEMPLATE;
		}
		return template;
	}

	private static UploadDoneDialog createUploadDoneDialog(Configuration options) {
		if (options.getSummaryTime() == null) {
			events.notifyError("summary display time not specified");
			return null;
		}
		return new UploadDoneDialog(new JFrame(), Integer.parseInt(options
				.getSummaryTime()));
	}

	private static PicUploaderTray createPicUploaderTray(String fileName, int number) {
		return new PicUploaderTray(fileName, number);
	}
	
	private static void setProxy(String proxyString) {
		final int portSeparatorPos = proxyString.lastIndexOf(':');
		if (portSeparatorPos == -1 || portSeparatorPos != proxyString.indexOf(':')) {
			events.notifyError("invalid proxy string: " + proxyString);
			return;
		}

		final Properties properties = System.getProperties();
		properties.put("proxySet", "true");
		properties.put("proxyHost", proxyString.substring(0, portSeparatorPos));
		properties.put("proxyPort", proxyString.substring(portSeparatorPos + 1));
	}

	public static void main(String[] args) {
		Configuration options = null;
		try {
			if(args[0].endsWith(".config")) {
			  options = new FileConfiguration(args);
			}
			else {
				options = new ArgumentConfiguration(args);
			}
		}catch(Exception e) {
			e.printStackTrace();
		}
		
		 

		events = new EventDispatcher();
		events.addEventListener(new DebugWriter());
		events.addEventListener(new ClipboardWriter(options.isVerboseSet(),
				createClipboardTemplate(options)));

		if (options.isLogFileSet()) {
			try {
				events.addEventListener(createLogfileWriter(options));
			}
			catch (IOException e) {
				events.notifyError("could not initialise logfile: "
						+ options.getLogFileName());
			}
		}

		if (options.isHtmlFileSet()) {
			try {
				events.addEventListener(createHtmlFileWriter(options));
			}
			catch (IOException e) {
				events.notifyError("could not initialise HTML file: "
						+ options.getHtmlFileName());
			}
		}

		if (options.isTrayiconSet()) {
    	  events.addEventListener(createPicUploaderTray(options.getTrayiconFile(), options.getFileNames().size()));
		}
		
		if (options.isSummarySet()) {
			try {
				events.addEventListener(createUploadDoneDialog(options));
			}
			catch (Exception e) {
				events.notifyError("could not initialise Dialog: " + e.getMessage());
			}
		}

		if (options.isSoundSet()) {
			if (!(new File(options.getSoundFile()).exists())) {
				events.notifyError("could not load sound file: "
						+ options.getSoundFile());
			}
			else {
				try {
					events.addEventListener(new SoundPlayer(options.getSoundFile()));
				}
				catch (Exception e) {
					events.notifyError("could not create sound clip: " + e.getMessage());
				}
			}
		}

		if (options.isProxySet()) {
			setProxy(options.getProxy());
		}

		try {
			if(!options.isScreenshotSet()) {
			  new PicUploader(events).uploadFiles(options.getFileNames(), options
					.isAutoscaleSet(), options.getMaxFileSize(), options);
			}//if
			else {
				String imageName = makeScreenshot(options.getScreenshotDelay(), options.isShowGUISet());
				if(imageName!=null) {
				  ArrayList l = new ArrayList();
				  l.add(imageName);
				    new PicUploader(events).uploadFiles(l, options
					  	.isAutoscaleSet(), options.getMaxFileSize(), options);
				  File f = new File((String)l.get(0));
				  f.delete();
				}//if imageName
				else {
					System.exit(0);
				}
			}//else
		}//try
		catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public static String makeScreenshot(int delay, boolean showGUI) {
		BufferedImage img = null;
		Toolkit toolkit = Toolkit.getDefaultToolkit();
		Dimension screenSize = toolkit.getScreenSize();
		try {
    		Thread.sleep(delay*1000);
			Robot r = new Robot();
			img = r.createScreenCapture(new Rectangle(0,0,(int)screenSize.getWidth(), (int)screenSize.getHeight()));
		}
		catch (Exception e) {
			e.printStackTrace();
		}
        if(showGUI) {
		  JDialog screenshotDialog = new JDialog();
		  ScreenshotCheckPanel panel = new ScreenshotCheckPanel(screenshotDialog);
		  panel.setImage(img);
		  screenshotDialog.setTitle("Drag mouse to select, CTRL + drag to censor, Shift + click to delete, ALT + drag to mark, ENTER to save and upload, press C to clear");
		  screenshotDialog.setModal(true);
		  screenshotDialog.getContentPane().add(new JScrollPane(panel));
          Insets insets = toolkit.getScreenInsets(GraphicsEnvironment
					.getLocalGraphicsEnvironment()
					.getDefaultScreenDevice()
					.getDefaultConfiguration());
		  screenshotDialog.setLocation(insets.left, insets.top);
          screenshotDialog.setSize((int)screenSize.getWidth()-insets.right, (int)screenSize.getHeight()-insets.bottom);
		  screenshotDialog.show();
		  return panel.getSavedImageName(); 
        }
        else {
        	try {
				File f = new File("./screenshots/~" + System.currentTimeMillis() + ".png");
				ImageIO.write(img, "png", f);
				return f.getAbsolutePath();
			}
			catch (Exception e) {
				e.printStackTrace();
			}
        }
		return null;
	}
}
