package com.nartosoft.picuploader;

import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.util.Iterator;
import java.util.Locale;

import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageWriter;
import javax.imageio.plugins.jpeg.JPEGImageWriteParam;
import javax.imageio.stream.ImageOutputStream;

/**
 * @author mf
 *  
 */

/**
 * Provides some static methods to make Image scaling more convenient
 */

public class PictureScaler {

	/**
	 * This class overrides the setCompressionQuality() method to workaround a
	 * problem in compressing JPEG images using the javax.imageio package.
	 * 
	 */
	static class MyImageWriteParam extends JPEGImageWriteParam {
		public MyImageWriteParam() {
			super(Locale.getDefault());
		}

		/**
		 * This method accepts quality levels between 0 (lowest) and 1 (highest) and
		 * simply converts it to a range between 0 and 256; this is not a correct
		 * conversion algorithm. However, a proper alternative is a lot more
		 * complicated. This should do until the bug is fixed.
		 */
		public void setCompressionQuality(float quality) {
			if (quality < 0.0F || quality > 1.0F) {
				throw new IllegalArgumentException("Quality out-of-bounds!");
			}
			this.compressionQuality = 256 - (quality * 256);
		}
	}
	
	/**
	 * Writes a given Image to a given Stream in a given Format
	 * 
	 * @param stream
	 * @param img
	 * @param fileFormat
	 * @throws Exception
	 */
	private static void writeToStream(ByteArrayOutputStream stream,
			BufferedImage img, String fileFormat) throws Exception {
		ImageWriter writer = null;
		Iterator iter = ImageIO.getImageWritersByFormatName(fileFormat);
		if (iter.hasNext()) {
			writer = (ImageWriter) iter.next();
		}
		ImageOutputStream ios = ImageIO.createImageOutputStream(stream);
		writer.setOutput(ios);
		// Set the compression quality
		ImageWriteParam iwparam = new MyImageWriteParam();
		iwparam.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
		iwparam.setCompressionQuality(0);

		// Write the image
		writer.write(null, new IIOImage(img, null, null), iwparam);
		// Cleanup
		writer.dispose();
		ios.flush();
		ios.close();
	}

	/**
	 * Write a BufferedImage to a File in the jpg Format using a quality setting
	 * of 100
	 * 
	 * @param f
	 * @param img
	 * @throws Exception
	 */
	private static void writeJpeg(File f, BufferedImage img) throws Exception {
		// Find a jpeg writer
		ImageWriter writer = null;
		Iterator iter = ImageIO.getImageWritersByFormatName("jpg");
		if (iter.hasNext()) {
			writer = (ImageWriter) iter.next();
		}
		// Prepare output file
		ImageOutputStream ios = ImageIO.createImageOutputStream(f);
		writer.setOutput(ios);
		// Set the compression quality
		ImageWriteParam iwparam = new MyImageWriteParam();
		iwparam.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
		iwparam.setCompressionQuality(0);
		// Write the image
		writer.write(null, new IIOImage(img, null, null), iwparam);
		// Cleanup
		ios.flush();
		writer.dispose();
		ios.close();
	}

	/**
	 * Write a given BufferedImage to the given File, deals with different Formats
	 */
	public static void writeToFile(File f, BufferedImage img) throws Exception {
		String fileFormat = Utils.getFileExtension(f.getName());
		if (fileFormat.equalsIgnoreCase("jpg") || fileFormat.equalsIgnoreCase("jpeg")) {
			writeJpeg(f, img);
		}
		else {
			ImageIO.write(img, fileFormat, f);
		}
	}

	/**
	 * Resizes a given BufferedImage to <= targetSize. Tries to guess how much to
	 * resize, guessing needs some work, still, works fine enough with 666kb.
	 * 
	 * @param image
	 * @param initialSize
	 * @param targetSize
	 * @return
	 */
	private static BufferedImage resizeImage(BufferedImage image,
			long initialSize, int targetSize) {

		float size = targetSize * 1024;
		float factor = (float) ((size / initialSize) * 1.7);
		while (factor > 1) {
			factor = (float) (factor * 0.75);
		}

		BufferedImage newImage = new BufferedImage(
				(int) (image.getWidth() * factor), (int) (image.getHeight() * factor),
				BufferedImage.TYPE_INT_RGB);
		newImage.getGraphics().drawImage(
				image.getScaledInstance((int) (image.getWidth() * factor), (int) (image
						.getHeight() * factor), BufferedImage.SCALE_SMOOTH), 0, 0,
				(int) (image.getWidth() * factor), (int) (image.getHeight() * factor),
				null);
		return newImage;
	}

	/**
	 * Tries to load the Image given in fileName and scales it to <=targetSizeKB
	 * 
	 * @param fileName
	 * @param targetSizeKB
	 * @return
	 */
	public static String scaleImage(String fileName, int targetSizeKB) {
		return scaleImage(new File(fileName), targetSizeKB).getAbsolutePath();
	}
	
	public static File scaleImage(File imageFile, int targetSizeKB) {
		if (imageFile.length() <= targetSizeKB) {
			return imageFile;
		}
		try {
			BufferedImage img = ImageIO.read(imageFile);
			return new File(scaleImage(img, imageFile.getAbsolutePath(), targetSizeKB));
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Takes a BufferedImage and, after having it resized to <=targetSizeKB writes
	 * it to a file with the name of scaled_oldFileName. If no scaling has to be
	 * done just returns original filename
	 * 
	 * @param img
	 * @param fileName
	 * @param targetSizeKB
	 * @return
	 * @throws Exception
	 */
	private static String scaleImage(BufferedImage img, String fileName,
			int targetSizeKB) throws Exception {
		
		// TODO: we know the initial file size from the input file, but here we recalculate it
		ByteArrayOutputStream initStream = new ByteArrayOutputStream();
		writeToStream(initStream, img, Utils.getFileExtension(fileName));
		long imageSize = initStream.toByteArray().length;
		if (imageSize < targetSizeKB * 1024) {
			return fileName;
		}
		
		ByteArrayOutputStream stream = null;
		while (imageSize >= targetSizeKB * 1024) {
			stream = new ByteArrayOutputStream();
			img = resizeImage(img, imageSize, targetSizeKB);
			writeToStream(stream, img, Utils.getFileExtension(fileName));
			imageSize = stream.toByteArray().length;
			//System.out.println("size: " + imageSize);
		}
		int index = fileName.lastIndexOf("\\");
		String uploadName = fileName;
		if (index != -1) {
			uploadName = fileName.substring(0, index + 1) + "scaled_"
					+ fileName.substring(index + 1, fileName.length());
			writeToFile(new File(uploadName), img);
		}
		else {
			uploadName = "scaled_" + fileName;
			writeToFile(new File(uploadName), img);

		}
		return uploadName;
	}

	public static void main(String[] args) {
		try {
			System.out
					.println(PictureScaler
							.scaleImage(
									"C:\\Dokumente und Einstellungen\\michael.frank\\workspace\\Uploadertests\\test.jpg",
									666));
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
}
